@@ -10,14 +10,13 @@
 struct drm_i915_private;
 
 #ifdef __linux__
-int i915_gemfs_init(struct drm_i915_private *i915);
-
+void i915_gemfs_init(struct drm_i915_private *i915);
 void i915_gemfs_fini(struct drm_i915_private *i915);
 #elif defined(__FreeBSD__)
-static inline int
+static inline void
 i915_gemfs_init(struct drm_i915_private *i915)
 {
-	return -ENODEV;
+	return;
 }
 
 static inline void
